;;; latex-tempo.el --- Templates for LaTeX with help of tempo.el and AucTeX

;; Author: Guy Yeterian
;;
;; Created: 01 July 1998
;; Version: 1.0
;; Version: 1.1 January 99
;; Keywords: Templates for LaTeX Mode with auctex

;; Copyright (C) 1998  GYe

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:
;;  This package is for use with AucTeX. You can add your TeX macros or add
;;  template for AucTeX.
;;  In version 1.1 we modify (with help of advice.el) tempo-insert. This
;;  function can now run TeX-insert-macro (look at the code for see how to do
;;  this.
;;

;;

;;; Description:
;;   latex-tempo.el  makes it easier to write LaTeX documents. This mode
;;   handles inserting LaTeX codes in a variety of ways (keybindings,
;;   completion in the buffer, running auctex functions both for command and
;;   environment ).





(defvar latex-tempo-env-tags nil
  "List of LaTeX environments tags.")

(defvar latex-tempo-com-tags nil
  "List of LaTeX commands tags.")


;;; latex-mode-hook
(require 'tempo)
(require 'advice)

(setq-default tempo-interactive t)
;(setq-default tempo-insert-region t)
(setq-default
 tempo-match-finder "\\b\\([^\b]+\\)\\=")  ;; The definition in tempo.el is false.

;;; install latex-mode and define list for environements and commands.
(add-hook 'LaTeX-mode-hook
	  '(lambda ()
	     (tempo-use-tag-list 'latex-tempo-env-tags)
	     (tempo-use-tag-list 'latex-tempo-com-tags)
	     ))

(add-hook 'LaTeX-math-mode-hook
 '(lambda ()
    (tempo-use-tag-list 'latex-tempo-env-tags)
    (tempo-use-tag-list	'latex-tempo-com-tags)
    ))

;;
;; modification of tempo for running auctex function
(defadvice tempo-insert (before run-function (element on-region))
  "run (auctex) lisp  function"
  (if (and (consp element)
	   (eq (car element) 'f))
      (progn
	(apply (nth 1 element) (nthcdr 2 element))
	(ad-set-arg 0 nil))))

;;
;; activate the new tempo-insert
(ad-activate 'tempo-insert t)


;;; LaTeX function uses later

;;
;; Insert a environment of the amsthm package.
;; pour completer mes macros persos sous amsthm
(defun LaTeX-env-thm (environment)
  "Insert ENVIRONMENT with optional argument
   usage with AMSTHM ."
  (let ((titre (read-string "Title : ")))
    (LaTeX-insert-environment environment
			      (concat
			       (if (not (zerop(length titre)))
				   (format "[%s]" titre))))))


;;
;;  tempo-user-elements
(defvar tempo-user-elements nil)


;;; tempo function's here.

;;
;;; function to construct the environments templates.
(defun latex-tempo-env (l)
  "Construct tempo-template for LaTeX environment"
  (let* ((tag (car l))
	 (nom (nth 1 l))
	 (element (nth 2 l)))
    (tempo-define-template nom element tag  nil 'latex-tempo-env-tags)))


;;; function to construct LaTeX commands
(defun latex-tempo-com (l)
  "Construct tempo-template for LaTeX commands."
  (let* ((tag (car l))
	 (element (nth 1 l)))
    (tempo-define-template tag element tag nil 'latex-tempo-com-tags)))




;;; The templates.

;;; here you install your templates for environments
;;; LaTeX environments tags
(mapcar
 'latex-tempo-env
 '(("enu" "enumerate"    ((LaTeX-env-item "enumerate")))
   ("ite" "itemize"      ((LaTeX-env-item "itemize")))
   ("des"  "description" ((LaTeX-env-item "description")))
   ("fig"  "figure"      ((LaTeX-env-item "figure")))
   ("pic"  "picture"    ((LaTeX-env-item "picture")))
   ("equ" "equation"    ((LaTeX-environment-menu "equation")))
   ("rem" "remarque"    ((LaTeX-environment-menu "remarque")))
   ("eno" "enonce"      ((LaTeX-environment-menu "enonce")))
   ("the" "theorem"     ((LaTeX-environment-menu "theorem")))
   ("pro" "proposition" ((LaTeX-environment-menu "proposition")))
   ("def"  "definition" ((LaTeX-environment-menu "definition")))
   ("cor"  "corollaire" ((LaTeX-environment-menu "corollaire")))
   ("exe"  "exercise"   ((LaTeX-environment-menu "exercise")))
   ("cla"  "claim"      ((LaTeX-environment-menu "claim")))
   ("tab"  "tabular"    ((LaTeX-environment-menu "tabular")))
   ("cen"  "center"     ((LaTeX-environment-menu "center")))
   ("eqn"  "eqnarray"   ((LaTeX-environment-menu "eqnarray*")))
   ("mini" "minipage" ((LaTeX-environment-menu "minipage")))
   ("mul"  "multicols" (& >"\\begin{multicols}{"(p "Nbre de cols :")"}%"n>
			  r >
			  n>"\\end{multicols}"> %))
   ("bmini"  "boxedminipage" (& >"\\begin{boxedminipage}["(p "Position :")"]{"(p "Largeur :")"}"n>
			  r >
			  n>"\\end{boxedminipage}"> %))
   ));;Here LaTeX environments tags



;;; Here you install your LaTeX commands tags
(mapcar
 'latex-tempo-com
   '(("int"  ("\\int_{" (p "liminf :") "}^{" (p "limsup :") "}" p
	       "\\;d"(p "variable :")))
     ("abs" ("\\left\\vert "p" \\right\\vert"p))
     ("norm" ("\\left\\Vert\\vec{"p"}\\right\\Vert"p))
     ("df" ("\\dfrac{"p"}{"p"}"p))
     ("nbr" ("nombre"))
     ("nbrc" ("nombre"p" complexe"p))
     ("nbrr" ("nombre"p" rel"p))
     ("bi" ("\\binom{"p"}{"p"}"p))
     ("fr" ("\\frac{"p"}{"p"}"p))
     ("sq" ("\\sqrt{"p"}"p))
     ("sqn" ("\\sqrt["p"]{"p"}"p))
     ("su" ("\\sum_{"p"}^{"p"}"p))
     ("coo" ("\\left("p"\\,;\\,"p"\\right)"p))
     ("coop" ("\\left["p"\\,;\\,"p"\\right]"p))
     ("coov" (& >"\\left(\\begin{array}[c]{c}%"n
		r >
		n"\\end{array}\\right)"> %))
     ("par" ("\\left("p"\\right)"p))
     ("cro" ("\\left["p"\\right]"p))
     ("acc" ("\\left\\{"p"\\right\\}"p))
     ("av" ("\\left(\\vec{"p"}\\,;\\,\\vec{"p"}\\right)"p))
     ("se" ("\\set{"p"}"p))
     ("ov" ("\\overline{"p"}"p))
     ("und" ("\\underline{"p"}"p))
     ("db" ("\\dbinom{"p"}{"p"}"p))
     ("te" ("\\text{"p"}"p))
     ("ve" ("\\vecteur{"p"}"p))
     ("mq" ("montrer que"))
     ("dmq" ("dmontrer que"))
     ("Mq" ("Montrer que"))
     ("Dmq" ("Dmontrer que"))
     ("cr" ("courbe reprsentative"))
     ("Et" ("\\'Etudier"))
     ("oij" ("(O\\,;\\,\\vec{\\imath},\\vec{\\jmath})"))
     ("ouv" ("(O\\,;\\,\\vec{u},\\vec{v})"))
     ("oijk" ("(O\\,;\\,\\vec{\\imath},\\vec{\\jmath},\\vec{k})"))
     ("got" ("\\textgoth{"p"}"p))
     ("fra" ("\\textfrak{"p"}"p))
     ("ssi" ("si, et seulement si, "))
     ("rp"  ("repre orthonorm "))
     ("act" ((act)))
     ("lv"  ((f TeX-insert-macro "limv")))
     ("ca"  ((f TeX-insert-macro "card")))
     ("fr"  ("\\frac{"p"}{"p"}"p))
     ("iff" ("\\left["p"\\,;\\,"p"\\right]"p))
     ("ifo" ("\\left["p"\\,;\\,"p"\\right["p))
     ("iof" ("\\left]"p"\\,;\\,"p"\\right]"p))
     ("ioo" ("\\left]"p"\\,;\\,"p"\\right["p))
     ("Df"  ("\\mathscr{D}_{f}"))
     ("Cf"  ("\\mathscr{C}_{f}"))
     ("tgt"  ("\\mathscr{T}_{"p"}"))
     ("cal"  ("\\mathscr{"p"}"p))
     ("ent"  ("\\Rightarrow "))
     ("equ" ("\\Leftrightarrow "))
     ("nent"  ("\\nRightarrow "))
     ("nequ" ("\\nLeftrightarrow "))
     ("gui"  ("\\og{}"p"\\fg{}"p))
     ("nn"  ("\\mathbb{N}"))
     ("nne"  ("\\mathbb{N}^{\\star}"))
     ("rr"  ("\\mathbb{R}"))
     ("rre"  ("\\mathbb{R}^{\\star}"))
     ("rrp"  ("\\mathbb{R}^{+}"))
     ("rrm"  ("\\mathbb{R}^{-}"))
     ("rrep"  ("\\mathbb{R}^{\\star+}"))
     ("rrem"  ("\\mathbb{R}^{\\star-}"))
     ("cc"  ("\\mathbb{C}"))
     ("cce"  ("\\mathbb{C}^\\star"))
     ("dd"  ("\\mathbb{D}"))
     ("pp"  ("\\mathscr{P}"))
     ("zz"  ("\\mathbb{Z}"))
     ("dd"  ("\\mathbb{D}"))
     ("qq"  ("\\mathbb{Q}"))
     ("eto"  ("^{\\star}"))
     ("etop"  ("^{\\star+}"))
     ("etom"  ("^{\\star-}"))
     ("qpq"  ("\\quad;\\quad"))
     ("qeq"  ("\\quad\\text{et}\\quad"))
     ("qoq"  ("\\quad\\text{o"p"}\\quad"))
     ("qpq"  ("\\quad;\\quad"))
     ("mpv"  ("\\,;\\,"))
     ("comp" ("\\makebox["(p "longueur (en mm):")"mm][l]{\\dotfill}\\;"))
     ("limx" ("\\underset{x\\rightarrow{}"p"}{\\lim}"p))
     ("limn" ("\\underset{n\\rightarrow+\\infty}{\\lim}"))
     ("lim" ("\\underset{"(p "Variable : ")"\\rightarrow{}"p"}{\\lim}"p))
     ("limxo" ("\\underset{x\\rightarrow0}{\\lim}"p))
     ("lims" ("\\underset{"(p "Variable : ")"\\stackrel{>}{\\rightarrow}"p"}{\\lim}"p))
     ("limi" ("\\underset{"(p "Variable : ")"\\stackrel{<}{\\rightarrow}"p"}{\\lim}"p))
     ("ten" ("\\xrightarrow[]{\\,"(p "Variable : ")"\\longrightarrow"p"\\,}"p))
     ("vs" ("\\vspace*{"(p "Longueur en mm: ")"mm}"))
     ("hs" ("\\hspace*{0mm}"))
     ("hf" ("\\hfill"))
     ("hfs" ("\\hfill\\hspace*{0mm}"))
     ("hsf" ("\\hspace*{0mm}\\hfill"))
     ("ct" ("\\cos\\theta"))
     ("st" ("\\sin\\theta"))
     ("ca" ("\\cos\\alpha"))
     ("sa" ("\\sin\\alpha"))
     ("sol" ("\\mathscr{S}=\\left\\{"p"\\right\\}"p))
     ("sol2" ("\\mathscr{S}=\\left\\{"p"\\,;\\,"p"\\right\\}"p))
     ("sol3" ("\\mathscr{S}=\\left\\{"p"\\,;\\,"p"\\,;\\,"p"\\right\\}"p))
     ("sol4" ("\\mathscr{S}=\\left\\{"p"\\,;\\,"p"\\,;\\,"p"\\,;\\,"p"\\right\\}"p))
     ("sol5" ("\\mathscr{S}=\\left\\{"p"\\,;\\,"p"\\,;\\,"p"\\,;\\,"p"\\,;\\,"p"\\right\\}"p))
     ("ens" ("\\left\\{"p"\\right\\}"p))
     ("ens2" ("\\left\\{"p"\\,;\\,"p"\\right\\}"p))
     ("ens3" ("\\left\\{"p"\\,;\\,"p"\\,;\\,"p"\\right\\}"p))
     ("ens4" ("\\left\\{"p"\\,;\\,"p"\\,;\\,"p"\\,;\\,"p"\\right\\}"p))
     ("ens5" ("\\left\\{"p"\\,;\\,"p"\\,;\\,"p"\\,;\\,"p"\\,;\\,"p"\\right\\}"p))
     ("vec"  ("\\vec{"r"}"p))
     ("aor"  ("\\left(\\vec{"r"}\\,,\\;\\vec{"r"}\\right)"p))
     ("aori"  ("\\left(\\vec{\\imath}\\,,\\;\\vec{"r"}\\right)"p))
     ("aoru"  ("\\left(\\vec{u}\\,,\\;\\vec{"r"}\\right)"p))
     ("pis"  ("\\frac{"(p "n.pi/q, n= ")"\\pi}{"(p "n.pi/q, q= ")"}"p))
     ("dpis"  ("\\dfrac{"(p "n.pi/q, n= ")"\\pi}{"(p "n.pi/q, q= ")"}"p))
     ("pinf"  ("+\\infty"))
     ("minf"  ("-\\infty"))
     ("sca"  ("\\vec{"p"}\\cdot\\vec{"p"}"p))
     ("tx"  ("\\text{"p"}"p))
     ("arg"  ("\\arg{"p"}"p))
     ("ass" ("\\longmapsto{}"))
     ("dp"  ("\\["p"\\]"))
     ("mod"  ("\\mod{2\\pi}"))
     ("ei"  ("\\e^{i"p"}"p))
     ("eipis"  ("\\e^{i\\frac{\pi}{"p"}"p))
     ("eia"  ("\\e^{i\\alpha}"))
     ("eib"  ("\\e^{i\\beta}"))
     ("eit"  ("\\e^{i\\theta}"))
     ("reia"  ("r\\e^{i\\alpha}"))
     ("reib"  ("r\\e^{i\\beta}"))
     ("reit"  ("r\\e^{i\\theta}"))
     ("colo"  ("\\colorize<"p">"p))
     ("coloi"  ("\\colorizeimp<"p">"p))
     ("colod"  ("\\colorizedef<"p">"p))
     ("som"  ("\\sum\\limits_{"p"}^{"p"}"))
     ("dis"  ("\\displaystyle"p))
     ("txt" ("\\text{"p"}"p))
     ("ince" ("\\subseteq{}"))
     ("inc" ("\\subset{}"))
     ("line" ("\\renewcommand{\\baselinestretch}{1.5}\\small\\normalsize"))
    ));;Here LaTeX commands tags

;; some local key.
;; The most important is f3 to complete your tag's.
(add-hook 'LaTeX-mode-hook
  (function
   (lambda ()
     (define-key LaTeX-mode-map [M-right] 'tempo-forward-mark)
     (define-key LaTeX-mode-map [M-left]  'tempo-backward-mark)
     (define-key LaTeX-mode-map [f3] 'tempo-complete-tag))))

(provide 'latex-tempo)
;;; latex-tempo ends here
